@@ -19,26 +19,25 @@
 import abc
 import itertools
 
-from oslo.config import cfg
-from stevedore import dispatch
-
 from ceilometer.openstack.common import context
 from ceilometer.openstack.common import log
 from ceilometer import pipeline
+from ceilometer import transformer
 
 LOG = log.getLogger(__name__)
 
 
 class PollingTask(object):
-    """Polling task for polling counters and inject into pipeline
-    A polling task can be invoked periodically or only once"""
+    """Polling task for polling samples and inject into pipeline.
+    A polling task can be invoked periodically or only once.
+
+    """
 
     def __init__(self, agent_manager):
         self.manager = agent_manager
         self.pollsters = set()
         self.publish_context = pipeline.PublishContext(
-            agent_manager.context,
-            cfg.CONF.counter_source)
+            agent_manager.context)
 
     def add(self, pollster, pipelines):
         self.publish_context.add_pipelines(pipelines)
@@ -46,19 +45,12 @@ class PollingTask(object):
 
     @abc.abstractmethod
     def poll_and_publish(self):
-        """Polling counter and publish into pipeline."""
+        """Polling sample and publish into pipeline."""
 
 
 class AgentManager(object):
 
     def __init__(self, extension_manager):
-        publisher_manager = dispatch.NameDispatchExtensionManager(
-            namespace=pipeline.PUBLISHER_NAMESPACE,
-            check_func=lambda x: True,
-            invoke_on_load=True,
-        )
-
-        self.pipeline_manager = pipeline.setup_pipeline(publisher_manager)
 
         self.pollster_manager = extension_manager
 
@@ -73,23 +65,28 @@ class AgentManager(object):
         for pipeline, pollster in itertools.product(
                 self.pipeline_manager.pipelines,
                 self.pollster_manager.extensions):
-            for counter in pollster.obj.get_counter_names():
-                if pipeline.support_counter(counter):
-                    polling_task = polling_tasks.get(pipeline.interval, None)
-                    if not polling_task:
-                        polling_task = self.create_polling_task()
-                        polling_tasks[pipeline.interval] = polling_task
-                    polling_task.add(pollster, [pipeline])
-                    break
+            if pipeline.support_meter(pollster.name):
+                polling_task = polling_tasks.get(pipeline.interval, None)
+                if not polling_task:
+                    polling_task = self.create_polling_task()
+                    polling_tasks[pipeline.interval] = polling_task
+                polling_task.add(pollster, [pipeline])
 
         return polling_tasks
 
     def initialize_service_hook(self, service):
+        self.pipeline_manager = pipeline.setup_pipeline(
+            transformer.TransformerExtensionManager(
+                'ceilometer.transformer',
+            ),
+        )
+
         self.service = service
         for interval, task in self.setup_polling_tasks().iteritems():
             self.service.tg.add_timer(interval,
                                       self.interval_task,
                                       task=task)
 
-    def interval_task(self, task):
+    @staticmethod
+    def interval_task(task):
         task.poll_and_publish()
