@@ -18,21 +18,15 @@
 
 from keystoneclient.v2_0 import client as ksclient
 from oslo.config import cfg
+from stevedore import extension
 
 from ceilometer import agent
-from ceilometer import extension_manager
 from ceilometer.openstack.common import log
-from ceilometer import service  # For cfg.CONF.os_*
-
-OPTS = [
-    cfg.ListOpt('disabled_central_pollsters',
-                default=[],
-                help='list of central pollsters to disable',
-                ),
-]
-
-cfg.CONF.register_opts(OPTS)
+from ceilometer.openstack.common import service as os_service
+from ceilometer.openstack.common.rpc import service as rpc_service
+from ceilometer import service
 
+cfg.CONF.import_group('service_credentials', 'ceilometer.service')
 
 LOG = log.getLogger(__name__)
 
@@ -41,13 +35,17 @@ class PollingTask(agent.PollingTask):
     def poll_and_publish(self):
         """Tasks to be run at a periodic interval."""
         with self.publish_context as publisher:
-            # TODO(yjiang5) passing counters into get_counters to avoid
+            # TODO(yjiang5) passing samples into get_samples to avoid
             # polling all counters one by one
+            cache = {}
             for pollster in self.pollsters:
                 try:
                     LOG.info("Polling pollster %s", pollster.name)
-                    publisher(list(pollster.obj.get_counters(
-                        self.manager)))
+                    samples = list(pollster.obj.get_samples(
+                        self.manager,
+                        cache,
+                    ))
+                    publisher(samples)
                 except Exception as err:
                     LOG.warning('Continue after error from %s: %s',
                                 pollster.name, err)
@@ -58,10 +56,10 @@ class AgentManager(agent.AgentManager):
 
     def __init__(self):
         super(AgentManager, self).__init__(
-            extension_manager.ActivatedExtensionManager(
+            extension.ExtensionManager(
                 namespace='ceilometer.poll.central',
-                disabled_names=cfg.CONF.disabled_central_pollsters,
-            ),
+                invoke_on_load=True,
+            )
         )
 
     def create_polling_task(self):
@@ -69,10 +67,20 @@ class AgentManager(agent.AgentManager):
 
     def interval_task(self, task):
         self.keystone = ksclient.Client(
-            username=cfg.CONF.os_username,
-            password=cfg.CONF.os_password,
-            tenant_id=cfg.CONF.os_tenant_id,
-            tenant_name=cfg.CONF.os_tenant_name,
-            auth_url=cfg.CONF.os_auth_url)
+            username=cfg.CONF.service_credentials.os_username,
+            password=cfg.CONF.service_credentials.os_password,
+            tenant_id=cfg.CONF.service_credentials.os_tenant_id,
+            tenant_name=cfg.CONF.service_credentials.os_tenant_name,
+            cacert=cfg.CONF.service_credentials.os_cacert,
+            auth_url=cfg.CONF.service_credentials.os_auth_url,
+            region_name=cfg.CONF.service_credentials.os_region_name,
+            insecure=cfg.CONF.service_credentials.insecure)
 
         super(AgentManager, self).interval_task(task)
+
+
+def agent_central():
+    service.prepare_service()
+    os_service.launch(rpc_service.Service(cfg.CONF.host,
+                                          'ceilometer.agent.central',
+                                          AgentManager())).wait()
