@@ -1,5 +1,10 @@
+# vim: tabstop=4 shiftwidth=4 softtabstop=4
+
 # Copyright 2013 Cloudbase Solutions Srl
 #
+# Author: Claudiu Belu <cbelu@cloudbasesolutions.com>
+#         Alessandro Pilotti <apilotti@cloudbasesolutions.com>
+#
 # Licensed under the Apache License, Version 2.0 (the "License"); you may
 # not use this file except in compliance with the License. You may obtain
 # a copy of the License at
@@ -22,11 +27,11 @@ import sys
 if sys.platform == 'win32':
     import wmi
 
-from oslo_config import cfg
-from oslo_log import log as logging
+from oslo.config import cfg
 
 from ceilometer.compute.virt import inspector
-from ceilometer.i18n import _
+from ceilometer.openstack.common.gettextutils import _
+from ceilometer.openstack.common import log as logging
 
 CONF = cfg.CONF
 LOG = logging.getLogger(__name__)
@@ -41,24 +46,19 @@ class UtilsV2(object):
     _VIRTUAL_SYSTEM_TYPE_REALIZED = 'Microsoft:Hyper-V:System:Realized'
 
     _PROC_SETTING = 'Msvm_ProcessorSettingData'
-    _MEMORY_SETTING = "Msvm_MemorySettingData"
     _SYNTH_ETH_PORT = 'Msvm_SyntheticEthernetPortSettingData'
     _ETH_PORT_ALLOC = 'Msvm_EthernetPortAllocationSettingData'
-    _PORT_ACL_SET_DATA = 'Msvm_EthernetSwitchPortAclSettingData'
     _STORAGE_ALLOC = 'Msvm_StorageAllocationSettingData'
     _VS_SETTING_DATA = 'Msvm_VirtualSystemSettingData'
+    _AGGREG_METRIC = 'Msvm_AggregationMetricDefinition'
     _METRICS_ME = 'Msvm_MetricForME'
-    _BASE_METRICS_VALUE = 'Msvm_BaseMetricValue'
 
     _CPU_METRIC_NAME = 'Aggregated Average CPU Utilization'
-    _MEMORY_METRIC_NAME = 'Aggregated Average Memory Utilization'
-    _NET_IN_METRIC_NAME = 'Filtered Incoming Network Traffic'
-    _NET_OUT_METRIC_NAME = 'Filtered Outgoing Network Traffic'
+    _NET_IN_METRIC_NAME = 'Aggregated Filtered Incoming Network Traffic'
+    _NET_OUT_METRIC_NAME = 'Aggregated Filtered Outgoing Network Traffic'
     # Disk metrics are supported from Hyper-V 2012 R2
     _DISK_RD_METRIC_NAME = 'Disk Data Read'
     _DISK_WR_METRIC_NAME = 'Disk Data Written'
-    _DISK_LATENCY_METRIC_NAME = 'Average Disk Latency'
-    _DISK_IOPS_METRIC_NAME = 'Average Normalized Disk Throughput'
 
     def __init__(self, host='.'):
         if sys.platform == 'win32':
@@ -90,22 +90,9 @@ class UtilsV2(object):
         cpu_metrics_def = self._get_metric_def(self._CPU_METRIC_NAME)
         cpu_metric_aggr = self._get_metrics(vm, cpu_metrics_def)
 
-        cpu_used = 0
-        if cpu_metric_aggr:
-            cpu_used = int(cpu_metric_aggr[0].MetricValue)
-
-        return (cpu_used,
+        return (long(cpu_metric_aggr[0].MetricValue) if cpu_metric_aggr else 0,
                 int(cpu_sd.VirtualQuantity),
-                int(vm.OnTimeInMilliseconds))
-
-    def get_memory_metrics(self, vm_name):
-        vm = self._lookup_vm(vm_name)
-        memory_def = self._get_metric_def(self._MEMORY_METRIC_NAME)
-        metric_memory = self._get_metrics(vm, memory_def)
-        memory_usage = 0
-        if metric_memory:
-            memory_usage = int(metric_memory[0].MetricValue)
-        return memory_usage
+                long(vm.OnTimeInMilliseconds))
 
     def get_vnic_metrics(self, vm_name):
         vm = self._lookup_vm(vm_name)
@@ -117,16 +104,12 @@ class UtilsV2(object):
 
         for port in ports:
             vnic = [v for v in vnics if port.Parent == v.path_()][0]
-
-            metric_value_instances = self._get_metric_value_instances(
-                port.associators(wmi_result_class=self._PORT_ACL_SET_DATA),
-                self._BASE_METRICS_VALUE)
-            metric_values = self._sum_metric_values_by_defs(
-                metric_value_instances, [metric_def_in, metric_def_out])
+            metric_values = self._get_metric_values(
+                port, [metric_def_in, metric_def_out])
 
             yield {
-                'rx_mb': metric_values[0],
-                'tx_mb': metric_values[1],
+                'rx_bytes': metric_values[0],
+                'tx_bytes': metric_values[1],
                 'element_name': vnic.ElementName,
                 'address': vnic.Address
             }
@@ -153,43 +136,16 @@ class UtilsV2(object):
                 'host_resource': host_resource
             }
 
-    def get_disk_latency_metrics(self, vm_name):
-        vm = self._lookup_vm(vm_name)
-        metric_latency_def = self._get_metric_def(
-            self._DISK_LATENCY_METRIC_NAME)
-
-        disks = self._get_vm_resources(vm, self._STORAGE_ALLOC)
-        for disk in disks:
-            metric_values = self._get_metric_values(
-                disk, [metric_latency_def])
-
-            yield {
-                'disk_latency': metric_values[0],
-                'instance_id': disk.InstanceID,
-            }
-
-    def get_disk_iops_count(self, vm_name):
-        vm = self._lookup_vm(vm_name)
-        metric_def_iops = self._get_metric_def(self._DISK_IOPS_METRIC_NAME)
-        disks = self._get_vm_resources(vm, self._STORAGE_ALLOC)
-
-        for disk in disks:
-            metric_values = self._get_metric_values(
-                disk, [metric_def_iops])
-
-            yield {
-                'iops_count': metric_values[0],
-                'instance_id': disk.InstanceID,
-            }
-
-    @staticmethod
-    def _sum_metric_values(metrics):
+    def _sum_metric_values(self, metrics):
         tot_metric_val = 0
         for metric in metrics:
-            tot_metric_val += int(metric.MetricValue)
+            tot_metric_val += long(metric.MetricValue)
         return tot_metric_val
 
-    def _sum_metric_values_by_defs(self, element_metrics, metric_defs):
+    def _get_metric_values(self, element, metric_defs):
+        element_metrics = element.associators(
+            wmi_association_class=self._METRICS_ME)
+
         metric_values = []
         for metric_def in metric_defs:
             if metric_def:
@@ -200,21 +156,6 @@ class UtilsV2(object):
                 metric_values.append(0)
         return metric_values
 
-    @staticmethod
-    def _get_metric_value_instances(elements, result_class):
-        instances = []
-        for el in elements:
-            associators = el.associators(wmi_result_class=result_class)
-            if associators:
-                instances.append(associators[0])
-
-        return instances
-
-    def _get_metric_values(self, element, metric_defs):
-        element_metrics = element.associators(
-            wmi_association_class=self._METRICS_ME)
-        return self._sum_metric_values_by_defs(element_metrics, metric_defs)
-
     def _lookup_vm(self, vm_name):
         vms = self._conn.Msvm_ComputerSystem(ElementName=vm_name)
         n = len(vms)
@@ -231,8 +172,7 @@ class UtilsV2(object):
             element.associators(
                 wmi_association_class=self._METRICS_ME), metric_def)
 
-    @staticmethod
-    def _filter_metrics(all_metrics, metric_def):
+    def _filter_metrics(self, all_metrics, metric_def):
         return [v for v in all_metrics if
                 v.MetricDefinitionId == metric_def.Id]
 
