@@ -15,12 +15,14 @@
 # under the License.
 
 import datetime
+from oslo.config import cfg
 
 from keystoneclient import exceptions
 import requests
 
 from ceilometer.central import plugin
-from ceilometer import counter
+from ceilometer import sample
+from ceilometer.openstack.common.gettextutils import _
 from ceilometer.openstack.common import log
 
 LOG = log.getLogger(__name__)
@@ -41,7 +43,7 @@ class KwapiClient(object):
         if self.token is not None:
             headers = {'X-Auth-Token': self.token}
         request = requests.get(probes_url, headers=headers)
-        message = request.json
+        message = request.json()
         probes = message['probes']
         for key, value in probes.iteritems():
             probe_dict = value
@@ -55,34 +57,37 @@ class _Base(plugin.CentralPollster):
     @staticmethod
     def get_kwapi_client(ksclient):
         """Returns a KwapiClient configured with the proper url and token."""
-        endpoint = ksclient.service_catalog.url_for(service_type='energy',
-                                                    endpoint_type='internalURL'
-                                                    )
+        endpoint = ksclient.service_catalog.url_for(
+            service_type='energy',
+            endpoint_type=cfg.CONF.service_credentials.os_endpoint_type)
         return KwapiClient(endpoint, ksclient.auth_token)
 
-    def iter_probes(self, ksclient):
+    CACHE_KEY_PROBE = 'kwapi.probes'
+
+    def _iter_probes(self, ksclient, cache):
         """Iterate over all probes."""
+        if self.CACHE_KEY_PROBE not in cache:
+            cache[self.CACHE_KEY_PROBE] = self._get_probes(ksclient)
+        return iter(cache[self.CACHE_KEY_PROBE])
+
+    def _get_probes(self, ksclient):
         try:
             client = self.get_kwapi_client(ksclient)
         except exceptions.EndpointNotFound:
             LOG.debug(_("Kwapi endpoint not found"))
             return []
-        return client.iter_probes()
+        return list(client.iter_probes())
 
 
-class KwapiPollster(_Base):
-    """Kwapi pollster derived from the base class."""
+class EnergyPollster(_Base):
+    """Measures energy consumption."""
 
-    @staticmethod
-    def get_counter_names():
-        return ['energy', 'power']
-
-    def get_counters(self, manager):
-        """Returns all counters."""
-        for probe in self.iter_probes(manager.keystone):
-            yield counter.Counter(
+    def get_samples(self, manager, cache):
+        """Returns all samples."""
+        for probe in self._iter_probes(manager.keystone, cache):
+            yield sample.Sample(
                 name='energy',
-                type=counter.TYPE_CUMULATIVE,
+                type=sample.TYPE_CUMULATIVE,
                 unit='kWh',
                 volume=probe['kwh'],
                 user_id=None,
@@ -92,9 +97,17 @@ class KwapiPollster(_Base):
                     probe['timestamp']).isoformat(),
                 resource_metadata={}
             )
-            yield counter.Counter(
+
+
+class PowerPollster(_Base):
+    """Measures power consumption."""
+
+    def get_samples(self, manager, cache):
+        """Returns all samples."""
+        for probe in self._iter_probes(manager.keystone, cache):
+            yield sample.Sample(
                 name='power',
-                type=counter.TYPE_GAUGE,
+                type=sample.TYPE_GAUGE,
                 unit='W',
                 volume=probe['w'],
                 user_id=None,
