@@ -2,6 +2,9 @@
 #
 # Copyright © 2012 eNovance <licensing@enovance.com>
 #
+# Copyright 2013 IBM Corp
+# All Rights Reserved.
+#
 # Author: Julien Danjou <julien@danjou.info>
 #
 # Licensed under the Apache License, Version 2.0 (the "License"); you may
@@ -20,7 +23,7 @@ from ceilometer.openstack.common import log
 from ceilometer.openstack.common import timeutils
 
 from ceilometer.central import plugin
-from ceilometer import counter
+from ceilometer import sample
 from ceilometer import nova_client
 
 
@@ -28,27 +31,32 @@ class FloatingIPPollster(plugin.CentralPollster):
 
     LOG = log.getLogger(__name__ + '.floatingip')
 
-    @staticmethod
-    def get_counter_names():
-        return ['ip.floating']
-
-    def get_counters(self, manager):
+    def _get_floating_ips(self):
         nv = nova_client.Client()
-        for ip in nv.floating_ip_get_all():
-            self.LOG.info("FLOATING IP USAGE: %s" % ip.address)
-            yield counter.Counter(
+        return nv.floating_ip_get_all()
+
+    def _iter_floating_ips(self, cache):
+        if 'floating_ips' not in cache:
+            cache['floating_ips'] = list(self._get_floating_ips())
+        return iter(cache['floating_ips'])
+
+    def get_samples(self, manager, cache):
+        for ip in self._iter_floating_ips(cache):
+            self.LOG.info("FLOATING IP USAGE: %s" % ip.ip)
+            # FIXME (flwang) Now Nova API /os-floating-ips can't provide those
+            # attributes were used by Ceilometer, such as project id, host.
+            # In this fix, those attributes usage will be removed temporarily.
+            # And they will be back after fix the Nova bug 1174802.
+            yield sample.Sample(
                 name='ip.floating',
-                type=counter.TYPE_GAUGE,
+                type=sample.TYPE_GAUGE,
                 unit='ip',
                 volume=1,
                 user_id=None,
-                project_id=ip.project_id,
+                project_id=None,
                 resource_id=ip.id,
                 timestamp=timeutils.utcnow().isoformat(),
                 resource_metadata={
-                    'address': ip.address,
-                    'fixed_ip_id': ip.fixed_ip_id,
-                    'host': ip.host,
-                    'pool': ip.pool,
-                    'auto_assigned': ip.auto_assigned
+                    'address': ip.ip,
+                    'pool': ip.pool
                 })
