@@ -37,9 +37,7 @@ metadata_headers = X-TEST
 
 from __future__ import absolute_import
 
-from oslo.config import cfg
-from stevedore import dispatch
-from swift.common.utils import split_path
+from swift.common.utils import split_path, get_logger
 import webob
 
 REQUEST = webob
@@ -57,34 +55,33 @@ except ImportError:
     # Swift <= 1.7.5 ... module exists and has class.
     from swift.common.middleware.proxy_logging import InputProxy
 
-from ceilometer import counter
+from ceilometer import sample
 from ceilometer.openstack.common import context
 from ceilometer.openstack.common import timeutils
 from ceilometer import pipeline
 from ceilometer import service
+from ceilometer import transformer
 
 
 class CeilometerMiddleware(object):
-    """
-    Ceilometer middleware used for counting requests.
-    """
+    """Ceilometer middleware used for counting requests."""
 
     def __init__(self, app, conf):
         self.app = app
+        self.logger = get_logger(conf, log_route='ceilometer')
 
         self.metadata_headers = [h.strip().replace('-', '_').lower()
                                  for h in conf.get(
                                      "metadata_headers",
                                      "").split(",") if h.strip()]
 
-        service.prepare_service()
-        publisher_manager = dispatch.NameDispatchExtensionManager(
-            namespace=pipeline.PUBLISHER_NAMESPACE,
-            check_func=lambda x: True,
-            invoke_on_load=True,
-        )
+        service.prepare_service([])
 
-        self.pipeline_manager = pipeline.setup_pipeline(publisher_manager)
+        self.pipeline_manager = pipeline.setup_pipeline(
+            transformer.TransformerExtensionManager(
+                'ceilometer.transformer',
+            ),
+        )
 
     def __call__(self, env, start_response):
         start_response_args = [None]
@@ -104,21 +101,27 @@ class CeilometerMiddleware(object):
                         bytes_sent += len(chunk)
                     yield chunk
             finally:
-                self.publish_counter(env,
-                                     input_proxy.bytes_received,
-                                     bytes_sent)
+                try:
+                    self.publish_sample(env,
+                                        input_proxy.bytes_received,
+                                        bytes_sent)
+                except Exception:
+                    self.logger.exception('Failed to publish samples')
 
         try:
             iterable = self.app(env, my_start_response)
         except Exception:
-            self.publish_counter(env, input_proxy.bytes_received, 0)
+            self.publish_sample(env, input_proxy.bytes_received, 0)
             raise
         else:
             return iter_response(iterable)
 
-    def publish_counter(self, env, bytes_received, bytes_sent):
+    def publish_sample(self, env, bytes_received, bytes_sent):
         req = REQUEST.Request(env)
-        version, account, container, obj = split_path(req.path, 1, 4, True)
+        try:
+            version, account, container, obj = split_path(req.path, 2, 4, True)
+        except ValueError:
+            return
         now = timeutils.utcnow().isoformat()
 
         resource_metadata = {
@@ -133,15 +136,12 @@ class CeilometerMiddleware(object):
                 resource_metadata['http_header_%s' % header] = req.headers.get(
                     header.upper())
 
-        with pipeline.PublishContext(
-                context.get_admin_context(),
-                cfg.CONF.counter_source,
-                self.pipeline_manager.pipelines,
-        ) as publisher:
+        with self.pipeline_manager.publisher(
+                context.get_admin_context()) as publisher:
             if bytes_received:
-                publisher([counter.Counter(
+                publisher([sample.Sample(
                     name='storage.objects.incoming.bytes',
-                    type='delta',
+                    type=sample.TYPE_DELTA,
                     unit='B',
                     volume=bytes_received,
                     user_id=env.get('HTTP_X_USER_ID'),
@@ -151,9 +151,9 @@ class CeilometerMiddleware(object):
                     resource_metadata=resource_metadata)])
 
             if bytes_sent:
-                publisher([counter.Counter(
+                publisher([sample.Sample(
                     name='storage.objects.outgoing.bytes',
-                    type='delta',
+                    type=sample.TYPE_DELTA,
                     unit='B',
                     volume=bytes_sent,
                     user_id=env.get('HTTP_X_USER_ID'),
@@ -162,6 +162,20 @@ class CeilometerMiddleware(object):
                     timestamp=now,
                     resource_metadata=resource_metadata)])
 
+            # publish the event for each request
+            # request method will be recorded in the metadata
+            resource_metadata['method'] = req.method.lower()
+            publisher([sample.Sample(
+                name='storage.api.request',
+                type=sample.TYPE_DELTA,
+                unit='request',
+                volume=1,
+                user_id=env.get('HTTP_X_USER_ID'),
+                project_id=env.get('HTTP_X_TENANT_ID'),
+                resource_id=account.partition('AUTH_')[2],
+                timestamp=now,
+                resource_metadata=resource_metadata)])
+
 
 def filter_factory(global_conf, **local_conf):
     conf = global_conf.copy()
