@@ -1,6 +1,10 @@
+# -*- encoding: utf-8 -*-
 #
-# Copyright 2013 Intel Corp.
-# Copyright 2013-2014 eNovance
+# Copyright © 2013 Intel Corp.
+# Copyright © 2013 eNovance
+#
+# Author: Yunhong Jiang <yunhong.jiang@intel.com>
+#         Julien Danjou <julien@danjou.info>
 #
 # Licensed under the Apache License, Version 2.0 (the "License"); you may
 # not use this file except in compliance with the License. You may obtain
@@ -15,38 +19,29 @@
 # under the License.
 
 import abc
-
-from oslo_log import log
-from oslo_utils import netutils
-import six
 from stevedore import driver
+from ceilometer.openstack.common import network_utils
 
 
-LOG = log.getLogger(__name__)
-
-
-def get_publisher(conf, url, namespace):
+def get_publisher(url, namespace='ceilometer.publisher'):
     """Get publisher driver and load it.
 
-    :param url: URL for the publisher
+    :param URL: URL for the publisher
     :param namespace: Namespace to use to look for drivers.
     """
-    parse_result = netutils.urlsplit(url)
+    parse_result = network_utils.urlsplit(url)
     loaded_driver = driver.DriverManager(namespace, parse_result.scheme)
-    return loaded_driver.driver(conf, parse_result)
+    return loaded_driver.driver(parse_result)
 
 
-@six.add_metaclass(abc.ABCMeta)
-class ConfigPublisherBase(object):
-    """Base class for plugins that publish data."""
+class PublisherBase(object):
+    """Base class for plugins that publish the sampler."""
 
-    def __init__(self, conf, parsed_url):
-        self.conf = conf
+    __metaclass__ = abc.ABCMeta
+
+    def __init__(self, parsed_url):
+        pass
 
     @abc.abstractmethod
-    def publish_samples(self, samples):
-        """Publish samples into final conduit."""
-
-    @abc.abstractmethod
-    def publish_events(self, events):
-        """Publish events into final conduit."""
+    def publish_samples(self, context, samples):
+        "Publish samples into final conduit."
