@@ -28,10 +28,6 @@ class LogStorage(base.StorageEngine):
     """Log the data
     """
 
-    def register_opts(self, conf):
-        """Register any configuration options used by this engine.
-        """
-
     def get_connection(self, conf):
         """Return a Connection instance based on the configuration settings.
         """
@@ -43,9 +39,9 @@ class Connection(base.Connection):
     """
 
     def __init__(self, conf):
-        return
+        pass
 
-    def upgrade(self, version=None):
+    def upgrade(self):
         pass
 
     def clear(self):
@@ -62,6 +58,15 @@ class Connection(base.Connection):
                  data['resource_id'],
                  data['counter_volume'])
 
+    def clear_expired_metering_data(self, ttl):
+        """Clear expired data from the backend storage system according to the
+        time-to-live.
+
+        :param ttl: Number of seconds to keep records for.
+
+        """
+        LOG.info("Dropping data with TTL %d", ttl)
+
     def get_users(self, source=None):
         """Return an iterable of user id strings.
 
@@ -77,8 +82,9 @@ class Connection(base.Connection):
         return []
 
     def get_resources(self, user=None, project=None, source=None,
-                      start_timestamp=None, end_timestamp=None,
-                      metaquery={}, resource=None):
+                      start_timestamp=None, start_timestamp_op=None,
+                      end_timestamp=None, end_timestamp_op=None,
+                      metaquery={}, resource=None, pagination=None):
         """Return an iterable of dictionaries containing resource information.
 
         { 'resource_id': UUID of the resource,
@@ -93,14 +99,17 @@ class Connection(base.Connection):
         :param project: Optional ID for project that owns the resource.
         :param source: Optional source filter.
         :param start_timestamp: Optional modified timestamp start range.
+        :param start_timestamp_op: Optional start time operator, like gt, ge.
         :param end_timestamp: Optional modified timestamp end range.
+        :param end_timestamp_op: Optional end time operator, like lt, le.
         :param metaquery: Optional dict with metadata to match on.
         :param resource: Optional resource filter.
+        :param pagination: Optional pagination query.
         """
         return []
 
     def get_meters(self, user=None, project=None, resource=None, source=None,
-                   metaquery={}):
+                   limit=None, metaquery={}, pagination=None):
         """Return an iterable of dictionaries containing meter information.
 
         { 'name': name of the meter,
@@ -114,32 +123,19 @@ class Connection(base.Connection):
         :param project: Optional ID for project that owns the resource.
         :param resource: Optional resource filter.
         :param source: Optional source filter.
+        :param limit: Maximum number of results to return.
         :param metaquery: Optional dict with metadata to match on.
+        :param pagination: Optional pagination query.
         """
         return []
 
-    def get_samples(self, event_filter):
+    def get_samples(self, sample_filter):
         """Return an iterable of samples as created by
         :func:`ceilometer.meter.meter_message_from_counter`.
         """
         return []
 
-    def get_volume_sum(self, event_filter):
-        """Return the sum of the volume field for the samples
-        described by the query parameters.
-        """
-
-    def get_volume_max(self, event_filter):
-        """Return the maximum of the volume field for the samples
-        described by the query parameters.
-        """
-
-    def get_event_interval(self, event_filter):
-        """Return the min and max timestamp for samples
-        matching the event_filter.
-        """
-
-    def get_meter_statistics(self, event_filter, period=None):
+    def get_meter_statistics(self, sample_filter, period=None, groupby=None):
         """Return a dictionary containing meter statistics.
         described by the query parameters.
 
@@ -159,3 +155,72 @@ class Connection(base.Connection):
           }
 
         """
+        return []
+
+    def get_alarms(self, name=None, user=None,
+                   project=None, enabled=None, alarm_id=None, pagination=None):
+        """Yields a lists of alarms that match filters
+        """
+        return []
+
+    def create_alarm(self, alarm):
+        """Create alarm.
+        """
+        return alarm
+
+    def update_alarm(self, alarm):
+        """update alarm
+        """
+        return alarm
+
+    def delete_alarm(self, alarm_id):
+        """Delete a alarm
+        """
+
+    def get_alarm_changes(self, alarm_id, on_behalf_of,
+                          user=None, project=None, type=None,
+                          start_timestamp=None, start_timestamp_op=None,
+                          end_timestamp=None, end_timestamp_op=None):
+        """Yields list of AlarmChanges describing alarm history
+
+        Changes are always sorted in reverse order of occurence, given
+        the importance of currency.
+
+        Segregation for non-administrative users is done on the basis
+        of the on_behalf_of parameter. This allows such users to have
+        visibility on both the changes initiated by themselves directly
+        (generally creation, rule changes, or deletion) and also on those
+        changes initiated on their behalf by the alarming service (state
+        transitions after alarm thresholds are crossed).
+
+        :param alarm_id: ID of alarm to return changes for
+        :param on_behalf_of: ID of tenant to scope changes query (None for
+                             administrative user, indicating all projects)
+        :param user: Optional ID of user to return changes for
+        :param project: Optional ID of project to return changes for
+        :project type: Optional change type
+        :param start_timestamp: Optional modified timestamp start range
+        :param start_timestamp_op: Optional timestamp start range operation
+        :param end_timestamp: Optional modified timestamp end range
+        :param end_timestamp_op: Optional timestamp end range operation
+        """
+        raise NotImplementedError('Alarm history not implemented')
+
+    def record_alarm_change(self, alarm_change):
+        """Record alarm change event.
+        """
+        raise NotImplementedError('Alarm history not implemented')
+
+    def record_events(self, events):
+        """Write the events.
+
+        :param events: a list of model.Event objects.
+        """
+        raise NotImplementedError('Events not implemented.')
+
+    def get_events(self, event_filter):
+        """Return an iterable of model.Event objects.
+
+        :param event_filter: EventFilter instance
+        """
+        raise NotImplementedError('Events not implemented.')
