@@ -15,7 +15,11 @@
 # License for the specific language governing permissions and limitations
 # under the License.
 
-from sqlalchemy import *
+from sqlalchemy import Float
+from sqlalchemy import Integer
+from sqlalchemy import MetaData
+from sqlalchemy import Table
+
 
 meta = MetaData()
 
