@@ -1,3 +1,4 @@
+# -*- encoding: utf-8 -*-
 #
 #
 # Licensed under the Apache License, Version 2.0 (the "License"); you may
@@ -12,12 +13,22 @@
 # License for the specific language governing permissions and limitations
 # under the License.
 
-import sqlalchemy as sa
+from sqlalchemy import MetaData
+from sqlalchemy import Index
+from ceilometer.storage.sqlalchemy.models import Meter
+
+meta = MetaData()
 
 
 def upgrade(migrate_engine):
-    meta = sa.MetaData(bind=migrate_engine)
-    meter = sa.Table('meter', meta, autoload=True)
-    index = sa.Index('idx_meter_rid_cname', meter.c.resource_id,
-                     meter.c.counter_name)
+    meta.bind = migrate_engine
+    index = Index('idx_meter_rid_cname', Meter.resource_id,
+                  Meter.counter_name)
     index.create(bind=migrate_engine)
+
+
+def downgrade(migrate_engine):
+    meta.bind = migrate_engine
+    index = Index('idx_meter_rid_cname', Meter.resource_id,
+                  Meter.counter_name)
+    index.drop(bind=migrate_engine)
