@@ -2,8 +2,10 @@
 #
 # Copyright © 2012 New Dream Network, LLC (DreamHost)
 # Copyright © 2013 Intel corp.
+# Copyright © 2013 eNovance
 #
 # Author: Yunhong Jiang <yunhong.jiang@intel.com>
+#         Julien Danjou <julien@danjou.info>
 #
 # Licensed under the Apache License, Version 2.0 (the "License"); you may
 # not use this file except in compliance with the License. You may obtain
@@ -22,17 +24,18 @@ import datetime
 import mock
 
 from stevedore import extension
-from stevedore import dispatch
 from stevedore.tests import manager as extension_tests
 
-from ceilometer import counter
+from ceilometer import sample
 from ceilometer import pipeline
+from ceilometer import agent
 from ceilometer.tests import base
+from ceilometer import transformer
 
 
-default_test_data = counter.Counter(
+default_test_data = sample.Sample(
     name='test',
-    type=counter.TYPE_CUMULATIVE,
+    type=sample.TYPE_CUMULATIVE,
     unit='',
     volume=1,
     user_id='test',
@@ -46,72 +49,73 @@ default_test_data = counter.Counter(
 class TestPollster:
     test_data = default_test_data
 
-    @classmethod
-    def get_counter_names(self):
-        return [self.test_data.name]
-
-    def get_counters(self, manager, instance=None):
-        self.counters.append((manager, instance))
+    def get_samples(self, manager, cache, instance=None):
+        self.samples.append((manager, instance))
         return [self.test_data]
 
 
 class TestPollsterException(TestPollster):
-    def get_counters(self, manager, instance=None):
+    def get_samples(self, manager, cache, instance=None):
         # Put an instance parameter here so that it can be used
         # by both central manager and compute manager
         # In future, we possibly don't need such hack if we
-        # combin the get_counters() function again
-        self.counters.append((manager, instance))
+        # combine the get_samples() function again
+        self.samples.append((manager, instance))
         raise Exception()
 
 
 class BaseAgentManagerTestCase(base.TestCase):
 
-    class PublisherClass():
-        def __init__(self):
-            self.counters = []
-
-        def publish_counters(self, ctxt, counter, source):
-            self.counters.extend(counter)
-
     class Pollster(TestPollster):
-        counters = []
+        samples = []
         test_data = default_test_data
 
     class PollsterAnother(TestPollster):
-        counters = []
-        test_data = default_test_data._replace(name='testanother')
+        samples = []
+        test_data = sample.Sample(
+            name='testanother',
+            type=default_test_data.type,
+            unit=default_test_data.unit,
+            volume=default_test_data.volume,
+            user_id=default_test_data.user_id,
+            project_id=default_test_data.project_id,
+            resource_id=default_test_data.resource_id,
+            timestamp=default_test_data.timestamp,
+            resource_metadata=default_test_data.resource_metadata)
 
     class PollsterException(TestPollsterException):
-        counters = []
-        test_data = default_test_data._replace(name='testexception')
+        samples = []
+        test_data = sample.Sample(
+            name='testexception',
+            type=default_test_data.type,
+            unit=default_test_data.unit,
+            volume=default_test_data.volume,
+            user_id=default_test_data.user_id,
+            project_id=default_test_data.project_id,
+            resource_id=default_test_data.resource_id,
+            timestamp=default_test_data.timestamp,
+            resource_metadata=default_test_data.resource_metadata)
 
     class PollsterExceptionAnother(TestPollsterException):
-        counters = []
-        test_data = default_test_data._replace(name='testexceptionanother')
+        samples = []
+        test_data = sample.Sample(
+            name='testexceptionanother',
+            type=default_test_data.type,
+            unit=default_test_data.unit,
+            volume=default_test_data.volume,
+            user_id=default_test_data.user_id,
+            project_id=default_test_data.project_id,
+            resource_id=default_test_data.resource_id,
+            timestamp=default_test_data.timestamp,
+            resource_metadata=default_test_data.resource_metadata)
 
     def setup_pipeline(self):
-        self.publisher = self.PublisherClass()
-        self.publisher_manager = dispatch.NameDispatchExtensionManager(
-            'fake',
-            check_func=lambda x: True,
-            invoke_on_load=False,
+        self.transformer_manager = transformer.TransformerExtensionManager(
+            'ceilometer.transformer',
         )
-        self.publisher_manager.extensions = [
-            extension.Extension(
-                'test_pub',
-                None,
-                None,
-                self.publisher,
-            ), ]
-        self.publisher_manager.by_name = dict(
-            (e.name, e)
-            for e
-            in self.publisher_manager.extensions)
-
         self.mgr.pipeline_manager = pipeline.PipelineManager(
             self.pipeline_cfg,
-            self.publisher_manager)
+            self.transformer_manager)
 
     def create_extension_manager(self):
         return extension_tests.TestExtensionManager(
@@ -143,7 +147,7 @@ class BaseAgentManagerTestCase(base.TestCase):
 
     @abc.abstractmethod
     def setup_manager(self):
-        """Setup subclass specific managers"""
+        """Setup subclass specific managers."""
 
     @mock.patch('ceilometer.pipeline.setup_pipeline', mock.MagicMock())
     def setUp(self):
@@ -155,15 +159,15 @@ class BaseAgentManagerTestCase(base.TestCase):
             'interval': 60,
             'counters': ['test'],
             'transformers': [],
-            'publishers': ["test_pub"],
+            'publishers': ["test"],
         }, ]
         self.setup_pipeline()
 
     def tearDown(self):
-        self.Pollster.counters = []
-        self.PollsterAnother.counters = []
-        self.PollsterException.counters = []
-        self.PollsterExceptionAnother.counters = []
+        self.Pollster.samples = []
+        self.PollsterAnother.samples = []
+        self.PollsterException.samples = []
+        self.PollsterExceptionAnother.samples = []
         super(BaseAgentManagerTestCase, self).tearDown()
 
     def test_setup_polling_tasks(self):
@@ -171,7 +175,8 @@ class BaseAgentManagerTestCase(base.TestCase):
         self.assertEqual(len(polling_tasks), 1)
         self.assertTrue(60 in polling_tasks.keys())
         self.mgr.interval_task(polling_tasks.values()[0])
-        self.assertEqual(self.publisher.counters[0], self.Pollster.test_data)
+        pub = self.mgr.pipeline_manager.pipelines[0].publishers[0]
+        self.assertEqual(pub.samples[0], self.Pollster.test_data)
 
     def test_setup_polling_tasks_multiple_interval(self):
         self.pipeline_cfg.append({
@@ -179,7 +184,7 @@ class BaseAgentManagerTestCase(base.TestCase):
             'interval': 10,
             'counters': ['test'],
             'transformers': [],
-            'publishers': ["test_pub"],
+            'publishers': ["test"],
         })
         self.setup_pipeline()
         polling_tasks = self.mgr.setup_polling_tasks()
@@ -194,7 +199,7 @@ class BaseAgentManagerTestCase(base.TestCase):
                 'interval': 10,
                 'counters': ['test_invalid'],
                 'transformers': [],
-                'publishers': ["test_pub"],
+                'publishers': ["test"],
             })
         polling_tasks = self.mgr.setup_polling_tasks()
         self.assertEqual(len(polling_tasks), 1)
@@ -206,7 +211,7 @@ class BaseAgentManagerTestCase(base.TestCase):
             'interval': 60,
             'counters': ['testanother'],
             'transformers': [],
-            'publishers': ["test_pub"],
+            'publishers': ["test"],
         })
         self.setup_pipeline()
         polling_tasks = self.mgr.setup_polling_tasks()
@@ -221,22 +226,40 @@ class BaseAgentManagerTestCase(base.TestCase):
                 'interval': 10,
                 'counters': ['testexceptionanother'],
                 'transformers': [],
-                'publishers': ["test_pub"],
+                'publishers': ["test"],
             },
             {
                 'name': "test_pipeline_2",
                 'interval': 10,
                 'counters': ['testexception'],
                 'transformers': [],
-                'publishers': ["test_pub"],
+                'publishers': ["test"],
             },
         ]
         self.mgr.pipeline_manager = pipeline.PipelineManager(
             self.pipeline_cfg,
-            self.publisher_manager)
+            self.transformer_manager)
 
         polling_tasks = self.mgr.setup_polling_tasks()
         self.assertEqual(len(polling_tasks.keys()), 1)
-        task = polling_tasks.get(10)
+        polling_tasks.get(10)
         self.mgr.interval_task(polling_tasks.get(10))
-        self.assertEqual(len(self.publisher.counters), 0)
+        pub = self.mgr.pipeline_manager.pipelines[0].publishers[0]
+        self.assertEqual(len(pub.samples), 0)
+
+    def test_agent_manager_initialize_service_hook(self):
+        mgr = agent.AgentManager(self.mgr.pollster_manager)
+        mgr.create_polling_task = mock.MagicMock()
+        service = mock.MagicMock()
+        mgr.initialize_service_hook(service)
+        self.assertTrue(service.tg.add_timer.called)
+
+    def test_manager_exception_persistency(self):
+        self.pipeline_cfg.append({
+            'name': "test_pipeline",
+            'interval': 60,
+            'counters': ['testanother'],
+            'transformers': [],
+            'publishers': ["test"],
+        })
+        self.setup_pipeline()
