@@ -15,19 +15,15 @@
 # WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 # License for the specific language governing permissions and limitations
 # under the License.
-"""Test listing raw events.
-"""
+"""Test statistics objects."""
 
 import datetime
-import logging
-import unittest
 
 from ceilometer.api.controllers import v2
-
-LOG = logging.getLogger(__name__)
+from ceilometer.tests import base
 
 
-class TestStatisticsDuration(unittest.TestCase):
+class TestStatisticsDuration(base.TestCase):
 
     def setUp(self):
         super(TestStatisticsDuration, self).setUp()
