@@ -17,29 +17,20 @@
 # under the License.
 """Tests for ceilometer/agent/manager.py
 """
-
-import datetime
-
 import mock
-from oslo.config import cfg
-from stevedore import extension
-from stevedore.tests import manager as extension_tests
-from stevedore import dispatch
 
 from ceilometer import nova_client
 from ceilometer.compute import manager
-from ceilometer import counter
-from ceilometer import pipeline
 from ceilometer.tests import base
-
 from tests import agentbase
 
 
-@mock.patch('ceilometer.pipeline.setup_pipeline', mock.MagicMock())
-def test_load_plugins():
-    mgr = manager.AgentManager()
-    assert list(mgr.pollster_manager), 'Failed to load any plugins'
-    return
+class TestManager(base.TestCase):
+
+    @mock.patch('ceilometer.pipeline.setup_pipeline', mock.MagicMock())
+    def test_load_plugins(self):
+        mgr = manager.AgentManager()
+        self.assertIsNotNone(list(mgr.pollster_manager))
 
 
 class TestRunTasks(agentbase.BaseAgentManagerTestCase):
@@ -50,6 +41,9 @@ class TestRunTasks(agentbase.BaseAgentManagerTestCase):
         setattr(instance, 'OS-EXT-STS:vm_state', state)
         return instance
 
+    def _raise_exception(self):
+        raise Exception
+
     def setup_manager(self):
         self.mgr = manager.AgentManager()
 
@@ -68,14 +62,23 @@ class TestRunTasks(agentbase.BaseAgentManagerTestCase):
     def test_notifier_task(self):
         self.mgr.setup_notifier_task()
         self.mgr.poll_instance(None, self.instance)
-        self.assertEqual(len(self.Pollster.counters), 1)
-        assert self.publisher.counters[0] == self.Pollster.test_data
+        self.assertEqual(len(self.Pollster.samples), 1)
+        pub = self.mgr.pipeline_manager.pipelines[0].publishers[0]
+        self.assertEqual(pub.samples[0], self.Pollster.test_data)
 
     def test_setup_polling_tasks(self):
         super(TestRunTasks, self).test_setup_polling_tasks()
-        self.assertTrue(self.Pollster.counters[0][1] is self.instance)
+        self.assertTrue(self.Pollster.samples[0][1] is self.instance)
 
     def test_interval_exception_isolation(self):
         super(TestRunTasks, self).test_interval_exception_isolation()
-        self.assertEqual(len(self.PollsterException.counters), 1)
-        self.assertEqual(len(self.PollsterExceptionAnother.counters), 1)
+        self.assertEqual(len(self.PollsterException.samples), 1)
+        self.assertEqual(len(self.PollsterExceptionAnother.samples), 1)
+
+    def test_manager_exception_persistency(self):
+        super(TestRunTasks, self).test_manager_exception_persistency()
+        self.stubs.Set(nova_client.Client, 'instance_get_all_by_host',
+                       lambda *x: self._raise_exception())
+        mgr = manager.AgentManager()
+        polling_task = manager.PollingTask(mgr)
+        polling_task.poll_and_publish()
