@@ -19,8 +19,10 @@
 """Tests for libvirt inspector.
 """
 
+from ceilometer.compute.virt import inspector as virt_inspector
 from ceilometer.compute.virt.libvirt import inspector as libvirt_inspector
 from ceilometer.tests import base as test_base
+import fixtures
 
 
 class TestLibvirtInspection(test_base.TestCase):
@@ -47,6 +49,22 @@ class TestLibvirtInspection(test_base.TestCase):
         dom_xml = """
              <domain type='kvm'>
                  <devices>
+                    <!-- NOTE(dprince): interface with no target -->
+                    <interface type='bridge'>
+                       <mac address='fa:16:3e:93:31:5a'/>
+                       <source bridge='br100'/>
+                       <model type='virtio'/>
+                       <address type='pci' domain='0x0000' bus='0x00' \
+                       slot='0x03' function='0x0'/>
+                    </interface>
+                    <!-- NOTE(dprince): interface with no mac -->
+                    <interface type='bridge'>
+                       <source bridge='br100'/>
+                       <target dev='foo'/>
+                       <model type='virtio'/>
+                       <address type='pci' domain='0x0000' bus='0x00' \
+                       slot='0x03' function='0x0'/>
+                    </interface>
                     <interface type='bridge'>
                        <mac address='fa:16:3e:71:ec:6d'/>
                        <source bridge='br100'/>
@@ -73,6 +91,14 @@ class TestLibvirtInspection(test_base.TestCase):
                        </filterref>
                        <alias name='net1'/>
                      </interface>
+                     <interface type='bridge'>
+                       <mac address='fa:16:3e:96:33:f0'/>
+                       <source bridge='qbr420008b3-7c'/>
+                       <target dev='vnet2'/>
+                       <model type='virtio'/>
+                       <address type='pci' domain='0x0000' bus='0x00' \
+                       slot='0x03' function='0x0'/>
+                    </interface>
                  </devices>
              </domain>
         """
@@ -82,11 +108,13 @@ class TestLibvirtInspection(test_base.TestCase):
                                                        3L, 4L, 0L, 0L))
         self.domain.interfaceStats('vnet1').AndReturn((5L, 6L, 0L, 0L,
                                                        7L, 8L, 0L, 0L))
+        self.domain.interfaceStats('vnet2').AndReturn((9L, 10L, 0L, 0L,
+                                                       11L, 12L, 0L, 0L))
         self.mox.ReplayAll()
 
         interfaces = list(self.inspector.inspect_vnics(self.instance_name))
 
-        self.assertEquals(len(interfaces), 2)
+        self.assertEqual(len(interfaces), 3)
         vnic0, info0 = interfaces[0]
         self.assertEqual(vnic0.name, 'vnet0')
         self.assertEqual(vnic0.mac, 'fa:16:3e:71:ec:6d')
@@ -115,6 +143,16 @@ class TestLibvirtInspection(test_base.TestCase):
         self.assertEqual(info1.tx_bytes, 7L)
         self.assertEqual(info1.tx_packets, 8L)
 
+        vnic2, info2 = interfaces[2]
+        self.assertEqual(vnic2.name, 'vnet2')
+        self.assertEqual(vnic2.mac, 'fa:16:3e:96:33:f0')
+        self.assertEqual(vnic2.fref, None)
+        self.assertEqual(vnic2.parameters, dict())
+        self.assertEqual(info2.rx_bytes, 9L)
+        self.assertEqual(info2.rx_packets, 10L)
+        self.assertEqual(info2.tx_bytes, 11L)
+        self.assertEqual(info2.tx_packets, 12L)
+
     def test_inspect_disks(self):
         dom_xml = """
              <domain type='kvm'>
@@ -138,10 +176,29 @@ class TestLibvirtInspection(test_base.TestCase):
 
         disks = list(self.inspector.inspect_disks(self.instance_name))
 
-        self.assertEquals(len(disks), 1)
+        self.assertEqual(len(disks), 1)
         disk0, info0 = disks[0]
         self.assertEqual(disk0.device, 'vda')
         self.assertEqual(info0.read_requests, 1L)
         self.assertEqual(info0.read_bytes, 2L)
         self.assertEqual(info0.write_requests, 3L)
         self.assertEqual(info0.write_bytes, 4L)
+
+
+class TestLibvirtInspectionWithError(test_base.TestCase):
+
+    def setUp(self):
+        super(TestLibvirtInspectionWithError, self).setUp()
+        self.inspector = libvirt_inspector.LibvirtInspector()
+        self.useFixture(fixtures.MonkeyPatch(
+            'ceilometer.compute.virt.libvirt.inspector.'
+            'LibvirtInspector._get_connection',
+            self._dummy_get_connection))
+
+    def _dummy_get_connection(*args, **kwargs):
+        raise Exception('dummy')
+
+    def test_inspect_unknown_error(self):
+
+        self.assertRaises(virt_inspector.InspectorException,
+                          self.inspector.inspect_cpus, 'foo')
