@@ -3,6 +3,9 @@
 #
 # Copyright © 2012 eNovance <licensing@enovance.com>
 #
+# Copyright 2013 IBM Corp
+# All Rights Reserved.
+#
 # Author: Julien Danjou <julien@danjou.info>
 #
 # Licensed under the Apache License, Version 2.0 (the "License"); you may
@@ -19,9 +22,9 @@
 
 import mock
 
-from ceilometer import nova_client
-from ceilometer.network import floatingip
 from ceilometer.central import manager
+from ceilometer.network import floatingip
+from ceilometer import nova_client
 from ceilometer.openstack.common import context
 from ceilometer.tests import base
 
@@ -42,16 +45,18 @@ class TestFloatingIPPollster(base.TestCase):
         ips = []
         for i in range(1, 4):
             ip = mock.MagicMock()
-            ip.address = '1.1.1.%d' % i
+            ip.id = i
+            ip.ip = '1.1.1.%d' % i
+            ip.pool = 'public'
             ips.append(ip)
         return ips
 
     # FIXME(dhellmann): Is there a useful way to define this
     # test without a database?
     #
-    # def test_get_counters_none_defined(self):
+    # def test_get_samples_none_defined(self):
     #     try:
-    #         list(self.pollster.get_counters(self.manager,
+    #         list(self.pollster.get_samples(self.manager,
     #                                         self.context)
     #              )
     #     except exception.NoFloatingIpsDefined:
@@ -59,18 +64,30 @@ class TestFloatingIPPollster(base.TestCase):
     #     else:
     #         assert False, 'Should have seen an error'
 
-    def test_get_counters_not_empty(self):
-        counters = list(self.pollster.get_counters(self.manager))
-        self.assertEqual(len(counters), 3)
-        addresses = [c.resource_metadata['address']
-                     for c in counters
-                     ]
-        self.assertEqual(addresses, ['1.1.1.1',
-                                     '1.1.1.2',
-                                     '1.1.1.3',
-                                     ])
+    def test_get_samples_not_empty(self):
+        samples = list(self.pollster.get_samples(self.manager, {}))
+        self.assertEqual(len(samples), 3)
+        # It's necessary to verify all the attributes extracted by Nova
+        # API /os-floating-ips to make sure they're available and correct.
+        self.assertEqual(samples[0].resource_id, 1)
+        self.assertEqual(samples[0].resource_metadata["address"], "1.1.1.1")
+        self.assertEqual(samples[0].resource_metadata["pool"], "public")
 
-    def test_get_counter_names(self):
-        counters = list(self.pollster.get_counters(self.manager))
-        self.assertEqual(set([c.name for c in counters]),
-                         set(self.pollster.get_counter_names()))
+        self.assertEqual(samples[1].resource_id, 2)
+        self.assertEqual(samples[1].resource_metadata["address"], "1.1.1.2")
+        self.assertEqual(samples[1].resource_metadata["pool"], "public")
+
+        self.assertEqual(samples[2].resource_id, 3)
+        self.assertEqual(samples[2].resource_metadata["address"], "1.1.1.3")
+        self.assertEqual(samples[2].resource_metadata["pool"], "public")
+
+    def test_get_meter_names(self):
+        samples = list(self.pollster.get_samples(self.manager, {}))
+        self.assertEqual(set([s.name for s in samples]),
+                         set(['ip.floating']))
+
+    def test_get_samples_cached(self):
+        cache = {}
+        cache['floating_ips'] = self.faux_get_ips(None)[:2]
+        samples = list(self.pollster.get_samples(self.manager, cache))
+        self.assertEqual(len(samples), 2)
