@@ -19,74 +19,84 @@
 
 import httplib2
 import json
-import os
 import random
 import socket
 import subprocess
-import tempfile
 import time
-import unittest
+
+from ceilometer.tests import base
 
 
-class BinDbsyncTestCase(unittest.TestCase):
+class BinTestCase(base.TestCase):
     def setUp(self):
-        self.tempfile = tempfile.mktemp()
+        super(BinTestCase, self).setUp()
+        self.tempfile = self.temp_config_file_path()
         with open(self.tempfile, 'w') as tmp:
-            tmp.write("[DEFAULT]\n")
-            tmp.write("database_connection=log://localhost\n")
+            tmp.write("[database]\n")
+            tmp.write("connection=log://localhost\n")
 
     def test_dbsync_run(self):
-        subp = subprocess.Popen(["../bin/ceilometer-dbsync",
+        subp = subprocess.Popen(['ceilometer-dbsync',
                                  "--config-file=%s" % self.tempfile])
         self.assertEqual(subp.wait(), 0)
 
-    def tearDown(self):
-        os.unlink(self.tempfile)
+    def test_run_expirer(self):
+        subp = subprocess.Popen(['ceilometer-expirer',
+                                 "--config-file=%s" % self.tempfile])
+        self.assertEqual(subp.wait(), 0)
 
 
-class BinSendCounterTestCase(unittest.TestCase):
+class BinSendCounterTestCase(base.TestCase):
     def setUp(self):
-        self.tempfile = tempfile.mktemp()
+        super(BinSendCounterTestCase, self).setUp()
+        self.tempfile = self.temp_config_file_path()
+        pipeline_cfg_file = self.path_get('etc/ceilometer/pipeline.yaml')
         with open(self.tempfile, 'w') as tmp:
             tmp.write("[DEFAULT]\n")
             tmp.write(
                 "rpc_backend=ceilometer.openstack.common.rpc.impl_fake\n")
             tmp.write(
-                "pipeline_cfg_file=../etc/ceilometer/pipeline.yaml\n")
+                "pipeline_cfg_file=%s\n" % pipeline_cfg_file)
 
     def test_send_counter_run(self):
-        subp = subprocess.Popen(["../bin/ceilometer-send-counter",
+        subp = subprocess.Popen([self.path_get('bin/ceilometer-send-counter'),
                                  "--config-file=%s" % self.tempfile,
                                  "--counter-resource=someuuid",
                                  "--counter-name=mycounter"])
         self.assertEqual(subp.wait(), 0)
 
-    def tearDown(self):
-        os.unlink(self.tempfile)
 
-
-class BinApiTestCase(unittest.TestCase):
+class BinApiTestCase(base.TestCase):
 
     def setUp(self):
+        super(BinApiTestCase, self).setUp()
         self.api_port = random.randint(10000, 11000)
         self.http = httplib2.Http()
-        self.tempfile = tempfile.mktemp()
+        self.tempfile = self.temp_config_file_path()
+        pipeline_cfg_file = self.path_get('etc/ceilometer/pipeline.yaml')
+        policy_file = self.path_get('tests/policy.json')
         with open(self.tempfile, 'w') as tmp:
             tmp.write("[DEFAULT]\n")
-            tmp.write(
-                "metering_api_port=%s\n" % self.api_port)
             tmp.write(
                 "rpc_backend=ceilometer.openstack.common.rpc.impl_fake\n")
-            tmp.write("database_connection=log://localhost\n")
             tmp.write(
                 "auth_strategy=noauth\n")
             tmp.write(
                 "debug=true\n")
-        self.subp = subprocess.Popen(["../bin/ceilometer-api",
+            tmp.write(
+                "pipeline_cfg_file=%s\n" % pipeline_cfg_file)
+            tmp.write(
+                "policy_file=%s\n" % policy_file)
+            tmp.write("[api]\n")
+            tmp.write(
+                "port=%s\n" % self.api_port)
+            tmp.write("[database]\n")
+            tmp.write("connection=log://localhost\n")
+        self.subp = subprocess.Popen(['ceilometer-api',
                                       "--config-file=%s" % self.tempfile])
 
     def tearDown(self):
-        os.unlink(self.tempfile)
+        super(BinApiTestCase, self).tearDown()
         self.subp.kill()
         self.subp.wait()
 
@@ -97,10 +107,11 @@ class BinApiTestCase(unittest.TestCase):
             try:
                 r, c = self.http.request(url, 'GET')
             except socket.error:
-                time.sleep(.3)
+                time.sleep(.5)
                 self.assertEqual(self.subp.poll(), None)
             else:
                 return r, c
+        return (None, None)
 
     def test_v1(self):
         response, content = self.get_response('v1/meters')
