@@ -19,6 +19,7 @@
 
 """Command line tool for creating test data for ceilometer.
 """
+from __future__ import print_function
 
 import argparse
 import datetime
@@ -27,8 +28,8 @@ import sys
 
 from oslo.config import cfg
 
-from ceilometer.collector import meter
-from ceilometer import counter
+from ceilometer.publisher import rpc
+from ceilometer import sample
 from ceilometer import storage
 from ceilometer.openstack.common import timeutils
 
@@ -38,56 +39,56 @@ def main():
 
     parser = argparse.ArgumentParser(
         description='generate metering data',
-        )
+    )
     parser.add_argument(
         '--interval',
         default=10,
         type=int,
         help='the period between events, in minutes',
-        )
+    )
     parser.add_argument(
         '--start',
         default=31,
         help='the number of days in the past to start timestamps',
-        )
+    )
     parser.add_argument(
         '--end',
         default=2,
         help='the number of days into the future to continue timestamps',
-        )
+    )
     parser.add_argument(
         '--type',
         choices=('gauge', 'cumulative'),
         default='gauge',
         help='counter type',
-        )
+    )
     parser.add_argument(
         '--unit',
         default=None,
         help='counter unit',
-        )
+    )
     parser.add_argument(
         '--project',
         help='project id of owner',
-        )
+    )
     parser.add_argument(
         '--user',
         help='user id of owner',
-        )
+    )
     parser.add_argument(
         'resource',
         help='the resource id for the meter data',
-        )
+    )
     parser.add_argument(
         'counter',
         help='the counter name for the meter data',
-        )
+    )
     parser.add_argument(
         'volume',
         help='the amount to attach to the meter',
         type=int,
         default=1,
-        )
+    )
     args = parser.parse_args()
 
     # Set up logging to use the console
@@ -119,7 +120,7 @@ def main():
     # Generate events
     n = 0
     while timestamp <= end:
-        c = counter.Counter(name=args.counter,
+        c = sample.Sample(name=args.counter,
                             type=args.type,
                             unit=args.unit,
                             volume=args.volume,
@@ -128,15 +129,16 @@ def main():
                             resource_id=args.resource,
                             timestamp=timestamp,
                             resource_metadata={},
+                            source='artificial',
                             )
-        data = meter.meter_message_from_counter(c,
-                                                cfg.CONF.metering_secret,
-                                                'artificial')
+        data = rpc.meter_message_from_counter(
+            c,
+            cfg.CONF.publisher_rpc.metering_secret)
         conn.record_metering_data(data)
         n += 1
         timestamp = timestamp + increment
 
-    print 'Added %d new events' % n
+    print('Added %d new events' % n)
 
     return 0
 
